/* Automatically generated by refactor.pl.
 *
 *
 * CMDNAME: userflags
 * CMDLEVEL: QCMD_AUTHED
 * CMDARGS: 2
 * CMDDESC: Shows or changes user flags.
 * CMDFUNC: csu_douserflags
 * CMDPROTO: int csu_douserflags(void *source, int cargc, char **cargv);
 * CMDHELP: Usage: USERFLAGS <flags>
 * CMDHELP: Changes your current user flags, where:
 * CMDHELP: flags - changes to apply, in the usual flag letters preceded by +/- format.
 * CMDHELP: Valid user flags are:
 * CMDHELP:  +c ACH'MENTS - enables achievements system - allows use of achievement commands
 * CMDHELP:                 and sends achievement messages.
 * CMDHELP:  +n NOTICE    - causes the bot to sent you NOTICEs.  If this flag is not set the
 * CMDHELP:                 bot will communicate using PRIVMSG.
 */

#include "../chanserv.h"
#include "../../lib/irc_string.h"
#include <stdio.h>
#include <string.h>

int csu_douserflags(void *source, int cargc, char **cargv) {
  nick *sender=source;
  reguser *rup=getreguserfromnick(sender), *target;
  int arg=0, wasorisoper;
  flag_t flagmask, changemask, oldflags;
  char flagbuf[30];

  if (!rup)
    return CMD_ERROR;
  
  if (cargc>0 && (*cargv[0]!='+' && *cargv[0]!='-')) {
    arg++;
    /* If the first char isn't a "change" character, it must specify a target */

    if (!(target=findreguser(sender,cargv[0])))
      return CMD_ERROR;

    if (target!=rup && !cs_privcheck(QPRIV_VIEWUSERFLAGS, sender)) {
      chanservstdmessage(sender, QM_NOACCESSONUSER, "userflags", cargv[0]);
      return CMD_ERROR;
    }
  } else {
    target=rup;
  }
    
  if (cargc>arg) {
    /* OK, now we have a changestring.. */
    if (target!=rup && !cs_privcheck(QPRIV_CHANGEUSERFLAGS, sender)) {
      /* Safe to use cargv[0] because if target != rup then first arg must have been the target name */
      chanservstdmessage(sender, QM_NOACCESSONUSER, "userflags", cargv[0]);
      return CMD_ERROR;
    }

    strcpy(flagbuf,printflags(target->flags, ruflags));
    oldflags=target->flags;

    changemask=QUFLAG_NOTICE;

    if (target==rup) {
      /* If you're changing yourself, you can give up the "status" flags and add/remove notice */
      changemask|=(target->flags & (QUFLAG_OPER | QUFLAG_DEV | QUFLAG_PROTECT | QUFLAG_HELPER | QUFLAG_ADMIN | QUFLAG_STAFF));
    }

    /* Warning, policy ahead */

    if (UHasStaffPriv(rup))
      changemask |= QUFLAG_PROTECT;

    if (UHasOperPriv(rup))
      changemask |= QUFLAG_PROTECT | QUFLAG_TRUST; /* QUFLAG_NOINFO; */

    if (UHasAdminPriv(rup))
      changemask |= (QUFLAG_OPER | QUFLAG_HELPER | QUFLAG_CLEANUPEXEMPT | QUFLAG_STAFF);
    
    if (UIsDev(rup))
      changemask=QUFLAG_ALL;
    
    if (time(NULL) > ACHIEVEMENTS_START) {
      changemask |= QUFLAG_ACHIEVEMENTS;
    }
    
    wasorisoper = UHasOperPriv(target);
    setflags(&target->flags, changemask, cargv[arg], ruflags, REJECT_NONE);
    wasorisoper |= UHasOperPriv(target);

    /* More policy */
    if (!UHasStaffPriv(target)) {
      target->flags &= ~QUFLAG_PROTECT;
    }

    cs_log(sender,"USERFLAGS #%s %s (%s -> %s)",target->username,cargv[arg],flagbuf,printflags(target->flags, ruflags));

    /* only warn about interesting changes */
    if((target->flags ^ oldflags) & ~(QUFLAG_NOTICE | QUFLAG_INFO | QUFLAG_TRUST | QUFLAG_ACHIEVEMENTS)) {
      chanservwallmessage("%s (%s) just used USERFLAGS on %s %s (%s -> %s)",sender->nick,rup->username,target->username,cargv[arg],flagbuf,printflags(target->flags,ruflags));

#ifdef AUTHGATE_WARNINGS
      if(wasorisoper)
        chanservsendmessage(sender, "WARNING FOR PRIVILEGED USERS: you MUST go to https://auth.quakenet.org and attempt to login as %s (with any password) to update the cache, otherwise their old credentials will be preserved in certain circumstances.",target->username);
#endif
    }
    csdb_updateuser(target);

/* Disabled this for now as live ircu doesn't support it - splidge
    if ((anp=findauthname(rup->ID)))
      localusersetaccountflags(anp, cs_accountflagmap(target));
 */
 
    chanservstdmessage(sender, QM_DONE);
  }

  if (cs_privcheck(QPRIV_VIEWUSERFLAGS, sender))
    flagmask=QUFLAG_ALL;
  else
    flagmask=QUFLAG_INFO | QUFLAG_NOTICE | QUFLAG_OPER | QUFLAG_HELPER | QUFLAG_DEV | QUFLAG_ADMIN | QUFLAG_STAFF | QUFLAG_ACHIEVEMENTS;
  
  chanservstdmessage(sender, QM_CURUSERFLAGS, target->username, printflagsornone(target->flags & flagmask, ruflags));

  return CMD_OK;
}
